% Script to run pre-calculations for wave kinematics.
% The kinematics are based on the use of JONSWAP specter, and linear wave
% theory. Sea surface, velocity and accelleration is calculated
% in time and space, and stored in matrices. The calculated data is then
% written to a gridwave file that is readable by USFOS software. the
% gridwave file is written in a separate script called write_grid.m
%

if ~exist('gridw')
  mkdir('gridw'); % create folder to store txt-files with wavecomponent input for usfos

end
% Create JONSWAP spectrum with parameters Tp, gamma,Hs as given below.

%INPUT, spectrum parameters:
sigmaA=0.07;
sigmaB=0.09;
sigma=0.08;
g=9.81;
Tp=17;
Hs=16.4;
gamma=3.3;
delta=0.036-(0.0056*(Tp/sqrt(Hs))); % factor for JONSWAP
gamma1=exp(3.483*(1-(0.1975*delta*(Tp^4)/(Hs^2)))); % factor for JONSWAP
alpha=5.061*(1-(0.287*log(gamma)))*(Hs^2)/(Tp^4); % factor for JONSWAP spectrum
omegap=2*pi/Tp; % peak frequency
omegalo=2*pi/50; %lowest wave frequency in the spectrum, lower cut off 
omegaup=2*pi/3; % highgest wave frequency in th spectrum, upper cut off
d=150; %water depth

% simulation parameters
init_t=500; %
time1=700; % 
time2=1500; % 
time3=1500; % 
time=200;

N1=[75,100,200,500]; % matrix containing number of components for the individual simulations
w1=zeros(max(N1),length(N1));
% Calculating wave component amplitude using concept with random interval
% frequency AND rayleigh distributed amplitude
% 

% calculate velocities and acceleration amp for each component: 
% for h=1:length(N1);
%     for j=1:N1(h);
%     u(j,h)=kn(j,h)*g*randAmp_N1(j,h)/w1(j,h);     % linear velocity amplitude for each component
%     a(j,h)=kn(j,h)*g*randAmp_N1(j,h); % horizontal acceleration amplitude for each component.
%     end
% end
% amplitudes above are sent to file writing script, and depth profiles for each horisontal position is calculated for each
% time step

% variables for depth, position and time are introduced, and the input for
% writing gridwave files are calculated: surface elevation, velocity
% profile, and acceleration profile

% create surface elevation process in time and space. necessary size of wave field to
% be calculated is depending on displacement of sway tower. Previous
% analysis in usfos shows displacement of top of tower to be 60+ m in x direction in
% extreme situations. it is decided to use a size of wave field 70 m in
% both x diretions, and 20 meters in both y directions. 140x40 square
% meters. 

% surface elevation is calculated for horizontal positions ranging from -70
% to 70 m in x-direction, and with a resolution of 0.5 m between each
% calculation i space. The surface elevation is calculatued in time ranging
% from t=500 to t=1500s, with timestep=0.1s.
% "elev" matrix contains the surface elevations in time and space. for each
% timestep, 'nx' elevations are calculated. each column represents elevation calcultion for one time step, "elev" will have size 280x10000 waves are modelled from 2D linear
% wave components, so elevation does not vary in y-direction.
xmin=-70;
xmax=70;
ymin=-20;
ymax=20;
dx=10; % change for resolution desicisions
nx=(xmax-xmin)/dx; %number of nodes in x direction
ny=1; % number of nodes in Y direction
dt=0.5
sampl=20
%  wave components: fft with random amp and random frequency
% elevation matrix, for each row(time step), elevation is calculated for 280 nodes in
% the x direction. calculations are done for 20 samples, generating 20
% gridwave files for each set of component amount. statistics for surface
% elevations are stored for each run.

% Calculating wave component amplitude using concept with random interval
% frequency AND rayleigh distributed amplitude
% 
for samp=1:sampl;
    for k=1:length(N1)
    delta_w1(k)=(omegaup-omegalo)/N1(k);
        for i=1:N1(k);
        w1(i,k)=omegalo+((i-1)*delta_w1(k))+(delta_w1(k)*rand); %amplitude selected at random within frequency interval of each component
            if w1(i,k)<=omegap;
        factor1(i,1)=exp((-1)*(((w1(i,k)/omegap)-1)^2)/(2*sigmaA*sigmaA));
           else
        factor1(i,1)=exp((-1)*(((w1(i,k)/omegap)-1)^2)/(2*sigmaB*sigmaB));
        
            end
        gammaf1(i,1)=gamma^factor1(i,1);
        S_N1(i,k)=alpha*g*g*(w1(i,k)^(-5))*exp((-1.25)*((w1(i,k)/omegap)^(-4)))*gammaf1(i,1);
        amp_N1(i,k)=sqrt(2*S_N1(i,k)*delta_w1(k)); %temporary amp
        randAmp_N1(i,k)=raylinv(rand,sqrt((amp_N1(i,k)^2)/2)); % component wave amplitude; randomization of amplitude based on rayleigh distribution
        Phase1(i,k)=rand*360; % random phase angle
        Period1(i,k)=2*pi/w1(i,k); % wave component period
        k_k1(i,k)=((w1(i,k))^2)/g; % wave number k for each component, deep water
        kn(i,k)=(w1(i,k)^2)/(g*tanh(k_k1(i,k)*d)); % corrected wavenumber; term tanh(k*h) is close to 1, yet it is introduced like this for a small correction of k for finite water depth.
    
        end
    end
    % calculate velocities and acceleration amp for each component: 
    for h=1:length(N1);
        for j=1:N1(h);
            u(j,h)=kn(j,h)*g*randAmp_N1(j,h)/w1(j,h);     % linear velocity amplitude for each component
           % a(j,h)=kn(j,h)*g*randAmp_N1(j,h); % horizontal acceleration amplitude for each component.
        end
    end
    
    
    
    for t=1:time2/dt;
        step=((t-1)*dt);
        for r=1:nx;
       % elev500(t,r)=sum(randAmp_N1(:,4).*cos(-w1(:,4).*step+(kn(:,4).*(xmin+(r*dx)))+(Phase1(:,4)*pi/180)));  %it works
       % elev75(t,r)=sum(randAmp_N1(:,1).*cos(-w1(:,1).*step+(kn(:,1).*(xmin+(r*dx)))+(Phase1(:,1)*pi/180)));   % send to file writing script as is, and assign each row for each time step
      %  elev100(t,r)=sum(randAmp_N1(:,2).*cos(-w1(:,2).*step+(kn(:,2).*(xmin+(r*dx)))+(Phase1(:,2)*pi/180)));
        elev200(t,r)=sum(randAmp_N1(:,3).*cos(-w1(:,3).*step+(kn(:,3).*(xmin+(r*dx)))+(Phase1(:,3)*pi/180)));
        end
    end
%surface statistics from a fixed point on gridwave. x=0

%     stat_kurt500(samp)=kurtosis(elev500(:,(nx/2)));
%     stat_kurt75(samp)=kurtosis(elev75(:,(nx/2)));
%     stat_kurt100(samp)=kurtosis(elev100(:,(nx/2)));
     stat_kurt200(samp)=kurtosis(elev200(:,(nx/2)));
%     
%     stat_mean500(samp)=mean(elev500(:,(nx/2)));
%     stat_mean75(samp)=mean(elev75(:,(nx/2)));
%     stat_mean100(samp)=mean(elev100(:,(nx/2)));
     stat_mean200(samp)=mean(elev200(:,(nx/2)));
%     
%     stat_skew500(samp)=skewness(elev500(:,(nx/2)));
%     stat_skew75(samp)=skewness(elev75(:,(nx/2)));
%     stat_skew100(samp)=skewness(elev100(:,(nx/2)));
     stat_skew200(samp)=skewness(elev200(:,(nx/2)));
%     
%     stat_max500(samp)=max(abs((elev500(:,(nx/2)))));
%     stat_max75(samp)=max(abs(elev75(:,(nx/2))));
%     stat_max100(samp)=max(abs(elev100(:,(nx/2))));
    stat_max200(samp)=max(abs(elev200(:,(nx/2))));
%     
%     stat_var500(samp)=var(elev500(:,(nx/2)));
%     stat_var75(samp)=var(elev75(:,(nx/2)));
%     stat_var100(samp)=var(elev100(:,(nx/2)));
    stat_var200(samp)=var(elev200(:,(nx/2)));    
    


   % for 20 samples, create 20 gridwave files 


    %[stretch]=gridwrite(elev75,xmin,xmax,dx,ny,nx,dt,ymin,ymax,u(:,1),a(:,1),Phase1(:,1),kn(:,1),d,samp,N1(:,1),w1(:,1));
   % gridwrite(elev200,xmin,xmax,dx,ny,nx,dt,ymin,ymax,u(:,1),a(:,1),Phase1(:,1),kn(:,1),d,samp,N1(:,1),w1(:,1));
    %gridwrite(elev,xmin,xmax,dx,ny,nx,dt,ymin,ymax,u(:,1),a(:,1),Phase1(:,1),kn(:,1),d,samp,N1(:,1),w1(:,1));
    gridwrite(elev200,xmin,xmax,dx,ny,nx,dt,ymin,ymax,u(:,3),Phase1(:,3),kn(:,3),d,samp,N1(:,3),w1(:,3));

end