% Create Jonswap spectrum with parameters
%Tp = 17s
% Hs= 16.4
% gamma = 3.3
% Frequency range from 0.125/s to 2.7/s


% other dependent parameters
% sigma, sigmaA, sigmaB, V, g, alpha
%alpha dependent on Tp and Hm0, which is known values. linear variations
%over Tp is assumed(Myrhaug 2007). The spectrum is based on the code given
%in the paper; USFOS hydrodynamics eq. (1.48)
clear;
if ~exist('randfreq')
  mkdir('randfreq'); % create folder to store txt-files with wavecomponent input for usfos

end


sigmaA=0.07;
sigmaB=0.09;
sigma=0.08;
g=9.81;
Tp=17;
Hs=16.4;
gamma=3.3;
delta=0.036-(0.0056*(Tp/sqrt(Hs))); % factor for JONSWAP
gamma1=exp(3.483*(1-(0.1975*delta*(Tp^4)/(Hs^2)))); % factor for JONSWAP
alpha=5.061*(1-(0.287*log(gamma)))*(Hs^2)/(Tp^4); % factor for JONSWAP spectrum
omegap=2*pi/Tp; % peak frequency
omegalo=2*pi/50; %lowest wave frequency in the spectrum 
omegaup=2*pi/3; % highgest wave frequency in th spectrum
N1=75; 
N2=100; %N1-N4 are number of wave components to be calculated
N3=200;
N4=1000;
% % Calculating a matrix of spectral periods used to calculate the wave
% % component amplitudes. The frequency interval for a given component is
% % constant, and set depending on amount of components calculated(75, 100,
% % 200). The wave frequency is selcted to be a random value within the given
% % frequency interval used to calculate the amplitude(ref. Shao/Faltinsen).
% % The individual wave components are also assigned a random phase angle,
% % and will increase the randomness of the components.
% 
delta_w1=(omegaup-omegalo)/N1;
delta_w2=(omegaup-omegalo)/N2; % frequency interval width corresponding to N# wave components/step length
delta_w3=(omegaup-omegalo)/N3;
delta_w4=(omegaup-omegalo)/N4;


Dir=0;
% Calculating the wave ampltidues for N1 = 75 components. Calculation based on method
% suggested in zhao's paper. The frequency, w1, to calculate the spectrum
% is selected randomly within the frequencyinterval dw for each component. 
for k=1:20;
    fname=sprintf('Spec75_%d.txt', k);
    FileID=fopen(fname, 'w');
    
    fprintf(FileID, '# Wave components for usfos userdefined specter');
    fprintf(FileID, '\n# \t Component \t Amp \t Period \t Dir \t Phase');
        for i=1:N1;
        w1(i,1)=omegalo+((i-1)*delta_w1)+(delta_w1*rand);
            if w1(i,1)<=omegap;
        factor1(i,1)=exp((-1)*(((w1(i,1)/omegap)-1)^2)/(2*sigmaA*sigmaA));
           else
        factor1(i,1)=exp((-1)*(((w1(i,1)/omegap)-1)^2)/(2*sigmaB*sigmaB));
        
            end
        gammaf1(i,1)=gamma^factor1(i,1);
        S_N1(i,1)=alpha*g*g*(w1(i,1)^(-5))*exp((-1.25)*((w1(i,1)/omegap)^(-4)))*gammaf1(i,1);
        Wavamp_N1(i,1)=sqrt(2*S_N1(i,1)*delta_w1);
        Phase1(i,1)=rand*360;
        Period1(i,1)=2*pi/w1(i,1);
    
        fprintf(FileID, '\n\t\t%.0f\t\t%.3f\t\t%.3f\t%.0f\t\t%.0f', i, Wavamp_N1(i,1), Period1(i,1), Dir, Phase1(i,1));
        end
        fprintf(FileID, '\n#');
    fclose(FileID);        
    movefile(fname,'randfreq');
end

%For N2 = 100 components, for 20 samples:
for k=1:20;
    fname=sprintf('Spec100_%d.txt', k);
    FileID=fopen(fname,'w');
    fprintf(FileID, '# Wave components for usfos userdefined specter');
    fprintf(FileID, '\n# \t Component \t Amp \t Period \t Dir \t Phase');
    for i=1:N2;
        w2(i,1)=omegalo+((i-1)*delta_w2)+(delta_w2*rand);
        if w2(i,1)<=omegap;
            factor2(i,1)=exp((-1)*(((w2(i,1)/omegap)-1)^2)/(2*sigmaA*sigmaA));
        else
            factor2(i,1)=exp((-1)*(((w2(i,1)/omegap)-1)^2)/(2*sigmaB*sigmaB));
        
        end
        gammaf2(i,1)=gamma^factor2(i,1);
        S_N2(i,1)=alpha*g*g*(w2(i,1)^(-5))*exp((-1.25)*((w2(i,1)/omegap)^(-4)))*gammaf2(i,1);
        Wavamp_N2(i,1)=sqrt(2*S_N2(i,1)*delta_w2);
        Phase2(i,1)=rand*360;
        Period2(i,1)=2*pi/w2(i,1);
        fprintf(FileID, '\n\t\t%.0f\t\t%.3f\t\t%.3f\t%.0f\t\t%.0f', i, Wavamp_N2(i,1), Period2(i,1), Dir, Phase2(i,1));

    end
    fprintf(FileID, '\n#');
    fclose(FileID);
    movefile(fname,'randfreq');
end

%200 Components

 for k=1:20;
     fname=sprintf('Spec200_%d.txt', k);
    FileID=fopen(fname,'w');
    fprintf(FileID, '# Wave components for usfos userdefined specter');
    fprintf(FileID, '\n# \t Component \t Amp \t Period \t Dir \t Phase');
    for i=1:N3;
    w3(i,1)=omegalo+((i-1)*delta_w3)+(delta_w3*rand);
    if w3(i,1)<=omegap;
        factor3(i,1)=exp((-1)*(((w3(i,1)/omegap)-1)^2)/(2*sigmaA*sigmaA));
    else
        factor3(i,1)=exp((-1)*(((w3(i,1)/omegap)-1)^2)/(2*sigmaB*sigmaB));
        
    end
    gammaf3(i,1)=gamma^factor3(i,1);
    S_N3(i,1)=alpha*g*g*(w3(i,1)^(-5))*exp((-1.25)*((w3(i,1)/omegap)^(-4)))*gammaf3(i,1);
    Wavamp_N3(i,1)=sqrt(2*S_N3(i,1)*delta_w3);
    Phase3(i,1)=rand*360;
    Period3(i,1)=2*pi/w3(i,1);
    fprintf(FileID, '\n\t\t%.0f\t\t%.3f\t\t%.3f\t%.0f\t\t%.0f', i, Wavamp_N3(i,1), Period3(i,1), Dir, Phase3(i,1));

    end
    fprintf(FileID, '\n#');
    fclose(FileID);
    movefile(fname,'randfreq');
end

%1000 components

for k=1:20;
   fname=sprintf('Spec1000_%d.txt', k);
   FileID=fopen(fname,'w');
    fprintf(FileID, '# Wave components for usfos userdefined specter');
    fprintf(FileID, '\n# \t Component \t Amp \t Period \t Dir \t Phase');
    for i=1:N4;
    w4(i,1)=omegalo+((i-1)*delta_w4)+(delta_w4*rand);
    if w4(i,1)<=omegap;
        factor4(i,1)=exp((-1)*(((w4(i,1)/omegap)-1)^2)/(2*sigmaA*sigmaA));
    else
        factor4(i,1)=exp((-1)*(((w4(i,1)/omegap)-1)^2)/(2*sigmaB*sigmaB));
        
    end
    gammaf4(i,1)=gamma^factor4(i,1);
    S_N4(i,1)=alpha*g*g*(w4(i,1)^(-5))*exp((-1.25)*((w4(i,1)/omegap)^(-4)))*gammaf4(i,1);
    Wavamp_N4(i,1)=sqrt(2*S_N4(i,1)*delta_w4);
    Phase4(i,1)=rand*360;
    Period4(i,1)=2*pi/w4(i,1);
    fprintf(FileID, '\n\t\t%.0f\t\t%.3f\t\t%.3f\t%.0f\t\t%.0f', i, Wavamp_N4(i,1), Period4(i,1), Dir, Phase4(i,1));

    end
    fprintf(FileID, '\n#');
    fclose(FileID);
   movefile(fname,'randfreq');
end

    
