function phat = ygumb(x,y,z)
%PLOTGUMB Plot data on Gumbel distribution paper.
%
% CALL:  phat = plotgumb(X)
%
%       phat = [a b] Parameters (see prbgumb) estimated from the plot by
%              least squares method 
%          X = data vector or matrix
%
% Example:
%   R=rndgumb(2,0,1,100);
%   phat=plotgumb(R),shg
%
% See also pdfgumb, prbgumb, rndgumb, fitgumb, momgumb

% Reference: 
%  Johnson  N.L., Kotz S. and Balakrishnan, N. (1994)
%  Continuous Univariate Distributions, Volume 2. Wiley. 


% rewritten ms 20.06.2000
figure(1);
F=edf(x,'wdata',true);
x = F.args(:);
F1 = F.data;
plot(x,-log(-log(F1)),'b.','markersize',12);
U=[ones(size(x)) x];
c=U\(-log(-log(F1)));
a=1/c(2);
b=-c(1)*a;
hold on
plot(x,U*c,'r--')
hold off
title(z)
xlabel(y)
ylabel('-log(-log(F))')
wafostamp;
if nargout > 0,
  phat=[a,b];
end
filnavn=strjoin(z);
saveas(figure(1),filnavn,'jpg');